@@ -1,5 +1,5 @@
 /* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2683 2018-12-05 08:44:20Z stephane $
+ * $Id: TabCanvas.cpp 2686 2018-12-05 16:40:06Z stephane $
  */
 
 #include "GMM.hpp"
@@ -15,6 +15,10 @@ TabCanvas::TabCanvas(const std::string & name) :
 	opc_register_pkg(cfg().get_str(prefix + "_opc_pkg_register") + "="),
 	tab_is_active(false),
 	export_image(cfg().get_bool(prefix + "_export_image")),
+	time_since_kiln_changed(std::time(nullptr)),
+	time_since_run_changed(std::time(nullptr)),
+	time_since_pkg_changed(std::time(nullptr)),
+	time_since_last_board(std::time(nullptr)),
 	timer_frequency_in_milliseconds(cfg().get_int(prefix + "_chart_redraw_ms")),
 	comm_handle(INVALID_HANDLE_VALUE),
 	current_kiln(0),
@@ -240,6 +244,24 @@ void TabCanvas::periodic_update(void)
 	{
 		auto rect = chart.getBounds();
 		chart.setImage(draw_chart(rect.getWidth(), rect.getHeight()));
+
+		std::string tooltip;
+
+		tooltip = "the kiln was last updated " + Lox::approximateTime(time_since_kiln_changed);
+		kiln_num_label	.setTooltip(tooltip);
+		kiln_num_editor	.setTooltip(tooltip);
+
+		tooltip = "the run was last updated " + Lox::approximateTime(time_since_run_changed);
+		run_label.setTooltip(tooltip);
+		run_editor.setTooltip(tooltip);
+
+		tooltip = "the package was last updated " + Lox::approximateTime(time_since_pkg_changed);
+		pkg_label.setTooltip(tooltip);
+		pkg_editor.setTooltip(tooltip);
+
+		tooltip = "the number of boards was last updated " + Lox::approximateTime(time_since_last_board);
+		run_number_of_boards_label.setTooltip(tooltip);
+		run_number_of_boards_editor.setTooltip(tooltip);
 	}
 
 	return;
@@ -647,6 +669,8 @@ void TabCanvas::device_thread(void)
 
 						run_moisture_bucket[idx] ++;
 						pkg_moisture_bucket[idx] ++;
+
+						time_since_last_board = std::time(nullptr);
 					}
 
 //					LOG_MSG(device_name << ": value=" << value << ", incrementing bucket #" << idx << " to " << pkg_moisture_bucket[idx]);
@@ -934,11 +958,21 @@ void TabCanvas::parse_opc_console_output(void)
 		}
 
 		// Note we want (and need) the key to include the trailing "=" character,
-		// otherwise we could get some false positives since "N75:10" would match "N75:1",
-		// but "N75:10=" wont ever be a match for "N75:1=".
+		// otherwise we could get some false positives since "N75:1" would match "N75:10",
+		// but "N75:1=" wont ever be a match for "N75:10=".
 		const std::string key = line.substr(0, pos + 1);
 		const int value = std::stoi(line.substr(pos + 1));
 
+		if (value <= 0)
+		{
+			/* One of two things have happened:
+			 * 1) The line is invalid and we haven't actually read a positive numeric value we can use.
+			 * 2) The RSLinx server has sent out an update where every variable is zero, which we see every
+			 * once in a while.  As Dom says, "Ya ignore them. We would never have any of those values as zero".
+			 */
+			 continue;
+		}
+
 //		LOG_MSG("prefix=" << prefix << ": -> idx=" << idx << ", key=" << key << " value=" << value);
 
 		if (key.find(opc_register_kiln)	!= std::string::npos)	new_kiln	= value;
@@ -1086,9 +1120,23 @@ void TabCanvas::update_kiln_run_pkg(void)
 	run_editor.setText(run_str);
 	pkg_editor.setText(pkg_str);
 
-	current_kiln = new_kiln;
-	current_run = new_run;
-	current_pkg = new_pkg;
+	if (current_kiln != new_kiln)
+	{
+		current_kiln = new_kiln;
+		time_since_kiln_changed = std::time(nullptr);
+	}
+
+	if (current_run != new_run)
+	{
+		current_run = new_run;
+		time_since_run_changed = std::time(nullptr);
+	}
+
+	if (current_pkg != new_pkg)
+	{
+		current_pkg = new_pkg;
+		time_since_pkg_changed = std::time(nullptr);
+	}
 
 	cfg().setValue(prefix + "_last_kiln", new_kiln);
 	cfg().setValue(prefix + "_last_run", new_run);
