@@ -1,5 +1,5 @@
 /* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.hpp 2682 2018-12-05 08:30:42Z stephane $
+ * $Id: TabCanvas.hpp 2684 2018-12-05 09:25:29Z stephane $
  */
 
 #pragma once
@@ -81,6 +81,11 @@ class TabCanvas : public Component, public Timer, public Thread
 
 		bool		export_image;
 
+		std::time_t time_since_kiln_changed;
+		std::time_t time_since_run_changed;
+		std::time_t time_since_pkg_changed;
+		std::time_t time_since_last_board;
+
 		/// Determines how often the timer should trigger.  This is used to redraw the chart and update the screen.
 		int			timer_frequency_in_milliseconds;
 
